class Value: ...
class Vector: ...

def f32_vector(data: list[float]) -> Value: ...
def u8_vector(data: list[int]) -> Value: ...
def binary_vector(data: list[int]) -> Value: ...
def f32_sparse_vector(data: dict[int, float]) -> Value: ...
def u8_sparse_vector(data: dict[int, int]) -> Value: ...
